package com.mortennobel.mandelbrot;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class Mandelbrot extends JFrame implements ChangeListener {
	private MandelbrotAnimation anim = new MandelbrotAnimation(this);
	private MandelbrotSetting settings = anim.getSetting();

	private JPanel subPanel = new JPanel(new GridLayout(0,1));
	private JPanel controlPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
	private JRadioButton java2dCheckbox = new JRadioButton("Java2D",true);
	private JRadioButton joglCheckbox = new JRadioButton("JOGL",false);
	private JComboBox iterations = new JComboBox(new Object[]{32,64,128,256,512,1024,2048,4096});
	private JComboBox movementSpeed = new JComboBox(new Object[]{2,4,6,8,10,15,20});

	private ButtonGroup buttonGroup = new ButtonGroup();
	private JLabel fpsLabel = new JLabel("");

	private MandelbrotJava2D mandelbrotJava2D = new MandelbrotJava2D(settings);
	private MandelbrotJOGL mandelbrotJOGL = new MandelbrotJOGL(settings);
	private MandelbrotRender currentRendere;

	public Mandelbrot() {
		setSize(800,800);
		getContentPane().setLayout(new BorderLayout());
		setRenderPanel(mandelbrotJava2D);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		getContentPane().add(subPanel, BorderLayout.SOUTH);

		buttonGroup.add(java2dCheckbox);
		buttonGroup.add(joglCheckbox);
		subPanel.add(fpsLabel);
		fpsLabel.setHorizontalAlignment(SwingConstants.CENTER);
		controlPanel.add(new JLabel("Render"));
		controlPanel.add(java2dCheckbox);
		controlPanel.add(joglCheckbox);
		controlPanel.add(new JLabel("# Iterations"));
		controlPanel.add(iterations);
		iterations.setSelectedItem(settings.getIterations());
		iterations.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int iter = (Integer) iterations.getSelectedItem();
				settings.setIterations(iter);
				mandelbrotJava2D.getStats().clear();
				mandelbrotJOGL.getStats().clear();
			}
		});
		controlPanel.add(new JLabel("Movement speed"));
		controlPanel.add(movementSpeed);
		movementSpeed.setSelectedItem(anim.getMovementSpeed());
		movementSpeed.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int speed = (Integer) movementSpeed.getSelectedItem();
				anim.setMovementSpeed(speed);
			}
		});

		subPanel.add(controlPanel);


		ActionListener al = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (e.getSource()==joglCheckbox){
					setRenderPanel(mandelbrotJOGL);
				} else {
					setRenderPanel(mandelbrotJava2D);
				}
			}
		};

		joglCheckbox.addActionListener(al);
		java2dCheckbox.addActionListener(al);
		anim.start();
		setVisible(true);
	}

	private void setRenderPanel(MandelbrotRender newRendere){
		assert(newRendere!=null);
		if (newRendere==currentRendere){
			return;
		}
		if (currentRendere!=null){
			currentRendere.pauseAnimation();
			getContentPane().remove(currentRendere.getComponent());
		}
		currentRendere = newRendere;
		currentRendere.resumeAnimation();
		getContentPane().add(currentRendere.getComponent(), BorderLayout.CENTER);
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new Mandelbrot();
			}
		});
	}

	private void updateFps(){
		float avarageRendertimeMillis = currentRendere.getStats().getAvarageRendertime();
		float fps = 1000/avarageRendertimeMillis;
		String fpsString = String.format("Rendertime (millis): %4.1f  FPS %4.1f",avarageRendertimeMillis,fps);
		fpsLabel.setText(fpsString);
	}

	public void stateChanged(ChangeEvent e) {
		if (currentRendere==mandelbrotJava2D){
			repaint();
		}
		updateFps();
	}
}
